.background, .background td
{
  font-size: 10pt;
}

.background
{
  background-color: #27A9FF;
  margin-top: 0;
  padding-top: 0;
  margin-bottom: 0;
  padding-bottom: 0;
}

.recipes
{
  background-repeat: repeat-y;
  padding-left: 70px;
  padding-top: 10px;
}

.background td
{
  background-color: inherit !important;
}

.ingredient-name
{
  font-weight: bold;
}

.ingredient-group
{
  font-weight: bold;
  text-decoration: underline;
}

h1
{
  font-size: 130%;
  border-bottom: 2px solid ridge;
  padding-top: 10px;
}

.header
{
  font-weight: bold;
}

.ingredient-amount-header, .ingredient-name-header, .ingredient-prep-method-header
{
  color: gray;
  text-align: center;
}

.ingredients td, .properties td
{
  border-style: none !important;
}

.ratings .comments
{
  font-style: italic;
}

.ratings .rater
{
  font-weight: bold;
}

.ratings .criteria-name
{
  font-size: small;
}

.title
{
  text-align: center;
  font-weight: bold;
  font-size: 150%;
}

.authors
{
  margin-top: 0px;
  text-align: center;
}
